/*
 file: win_authClient.cpp
 author: Nafees Bin Zafar <binzafar@musc.edu>
         Robert Gorlitsky <gorlitsk@musc.edu>
 date:   1/25/1999

 synopsis: Implementation of the NIS authentication portal.
           The only necessary interface should be to clientmain();
 */

#include "win_authClient.h"

/* clientmain:  Should be the only interface required
   host: (string) hostname
   uname: (string) username in host
   upass: (string) password in host
   result: (string) preallocated buffer where the GECOS common name of the user is stored
 */
void clientmain (char* host, int port, char* uname, char* upass, char* result) {
  char* usrinfo = new(char[MAXRESULT]);
  
  WSAData wsdat;
  SOCKET s;
  
#ifdef DEBUG
  cout << "DEBUG::clientmain(): host = " << host << "\tuname = " << uname << "\tpasswd = " << upass << endl;
  cout.flush();
#endif


  if (WSAStartup(MAKEWORD(1, 1), &wsdat) == 0) {
   memset (usrinfo, '\0', strlen(usrinfo));
   sprintf (usrinfo, "%s %s\n", uname, upass);
  
   s = getSock (host, port);

#ifdef DEBUG
   cout << "DEBUG::clientmain(): Socket successfully established = " << s << endl;
   cout.flush();
#endif

   authenticate (s, usrinfo, result);
   closeSock(s);
  
   WSACleanup();
  }
  
// return(result);
}

SOCKET getSock(char* uhost, int uport) {
  u_long host = getNetAddr (uhost);

  SOCKET s = socket (AF_INET, SOCK_STREAM, 0);
 
  if (s != INVALID_SOCKET) {
    sockaddr_in sinRemote;
    sinRemote.sin_family = AF_INET;
    sinRemote.sin_addr.s_addr = host;
    sinRemote.sin_port = htons(uport);
    if (connect(s, (sockaddr*)&sinRemote, sizeof(sockaddr_in))
	== SOCKET_ERROR) {
      s = INVALID_SOCKET;
    }
  }
  
  return s;
}


u_long getNetAddr (const char* uhost) {
  u_long host = inet_addr(uhost);
  if (host == INADDR_NONE) {
    // host isn't a dotted IP, so resolve it through DNS
    hostent* pHE = gethostbyname(uhost);
    if (pHE == 0) {
      return INADDR_NONE;
    }
    host = *((u_long*)pHE->h_addr_list[0]);
  }
  
  return host;
}


bool closeSock(SOCKET s) {
  if (shutdown(s, SD_SEND) == SOCKET_ERROR)
    return false;

  if (closesocket(s) == SOCKET_ERROR)
    return false;

  return true;
}


void authenticate (SOCKET s, const char* sendbuf, char* result) {
  const int buflen = strlen(sendbuf);
  char recvbuf[MAXRESULT];
  int i;

#ifdef DEBUG
  cout << "DEBUG::authenticate:sendbuf = " << sendbuf << endl;
  cout.flush();
#endif

	memset (recvbuf, 0, sizeof(recvbuf));

  if ((i = send(s, sendbuf, buflen + 1, 0)) != SOCKET_ERROR) {

#ifdef DEBUG
	  cout << "authenticate::send() succeeded = " << i << endl;
	  cout.flush();
#endif

	i = recv (s, recvbuf, sizeof(recvbuf), 0);

#ifdef DEBUG
	cout << "authenticate::recv() = " << i << endl;
	cout.flush();
#endif

	for (int i = 0; i < (int)strlen(recvbuf); i++)
		if (recvbuf[i] < 0)
			recvbuf[i] = '\0';
	strcpy (result, recvbuf);
  }
  else {
	cout << "authenticate::send() failed\n";
	cout.flush();
  }


//  return(recvbuf);
}
